@@ -67,6 +67,7 @@ use DBI                    ();
 use Time::HiRes            ();
 use Time::Elapsed          ();
 use File::Spec             ();
+use File::Next             ();
 use File::HomeDir          ();
 use File::ShareDir         ();
 use File::Find::Rule       ();
@@ -81,7 +82,7 @@ use PPI::Document          ();
 use PPI::Cache             ();
 use Module::Pluggable;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use constant ORLITE_FILE => File::Spec->catfile(
 	File::HomeDir->my_data,
@@ -103,6 +104,8 @@ use ORLite::Migrate 0.03 {
 	user_version => 3,
 };
 
+use Perl::Metrics2::CpanFile ();
+
 
 
 
@@ -121,7 +124,34 @@ sub new {
 		eval "require $plugin";
 		die $@ if $@;
 		$self->{plugins}->{$plugin} = $plugin->new;
-		$self->{plugins}->{$plugin}->study if $self->study;
+	}
+
+	# Study if needed.
+	if ( $self->study ) {
+		# If a document is present in all of the plugins
+		# use a more efficient single scalar.
+		my $all  = scalar keys %{$self->{plugins}};
+		my $sql = 'SELECT md5, package FROM file_metric';
+		my $sth = $self->prepare($sql) or die("prepare: $DBI::errstr");
+		$sth->execute or die("execute: $DBI::errstr");
+		my %seen = ();
+		while ( my $row = $sth->fetchrow_arrayref ) {
+			my $md5 = $row->[0];
+			my $pkg = $row->[1];
+			unless ( $seen{$md5} ) {
+				$seen{$md5} ||= {};
+			}
+			unless ( ref $seen{$md5} ) {
+				# All registered already
+				next;
+			}
+			$seen{$md5}->{$pkg} = 1;
+			if ( scalar keys %{$seen{$md5}} == $all ) {
+				$seen{$md5} = 1;
+			}
+		}
+		$sth->finish or die("finish: $DBI::errstr");
+		$self->{seen} = \%seen;
 	}
 
 	# Initialise the PPI cache if available
@@ -143,11 +173,20 @@ sub cache {
 sub seen {
 	my $self = shift;
 	my $md5  = shift;
-	foreach my $plugin ( sort keys %{$self->{plugins}} ) {
-		next if $self->{plugins}->{$plugin}->{seen}->{$md5};
-		return 0;
+	my $seen = $self->{seen}->{$md5};
+
+	# Document was seen by none
+	return 0 unless $seen;
+
+	# Document was seen by all
+	return 1 if not ref $seen;
+
+	# Seen by a specific plugin?
+	if ( @_ ) {
+		return 1 if $seen->{$_[0]};
 	}
-	return 1;
+
+	return 0;
 }
 
 
@@ -166,39 +205,7 @@ sub process_cache {
 		Carp::croak("Must have study true to process_cache");
 	}
 
-	# Find all the files in the cache
-	$self->trace("Scanning cache directory " . $self->cache . "...");
-	my @files = File::Find::Rule->name(qr/\.ppi\z/)->in($self->cache);
-	$self->trace("Found " . scalar(@files) . " documents");
-
-	# While filtering, save the total size of PPI storable
-	# content left to process.
-	my $total = 0;
-
-	# Filter and sort the documents
-	$self->trace("Cleaning, filtering and sorting documents...");
-	@files = map {
-		# Total the content
-		$total += $_->[2];
-
-		# Remove the schwartian
-		$_->[1]
-	} sort {
-		# Smallest files first (for lowest parser stress)
-		$a->[2] <=> $b->[2]
-	} grep {
-		# Filter out things we've done already
-		not $self->seen($_->[1])
-	} map {
-		# Set up for the Schwartzian transform
-		/([a-f0-9]+).ppi\z/ ? [ $_, "$1", (stat($_))[7] ] : ()
-	} @files;
-	$self->trace("Filtered to " . scalar(@files) . " documents");
-
-	# Shortcut if there's nothing to do
-	unless ( @files ) {
-		return 1;
-	}
+	$| = 1;
 
 	# Remove indexes to speed up inserts
 	$self->trace("Removing indexes for faster inserts...");
@@ -208,37 +215,42 @@ sub process_cache {
 		Perl::Metrics2->do($sql);
 	}
 
-	my $count = 0;
-	my $start = Time::HiRes::time();
-	my $rate  = 0;
-	my $left  = 0;
-	my $done  = 0;
-	my $cache = PPI::Document->get_cache;
+	# Find all the files in the cache
+	$self->trace("Processing cache directory " . $self->cache . "...");
 	$self->begin;
-	foreach my $md5 ( @files ) {
-		$self->trace(
-			sprintf(
-				"%s - %d of %d @ %.1fk/sec (%s remaining)",
-				$md5, ++$count, scalar(@files), $rate / 1024,
-				Time::Elapsed::elapsed($left),
-			)
-		);
+	my $count  = 0;
+	my $files  = 0;
+	my $cache  = PPI::Document->get_cache;
+	my $search = File::Next::files( {
+		sort_files => 1,
+	}, $self->cache );
+	while ( my @file = $search->() ) {
+		$file[1] =~ /([a-f0-9]+)\.ppi\z/ or next;
+		(++$files % 100) or print '.';
 
-		# Fetch the document from the cache and process it
+		# Filter out things we've done already
+		my $md5 = $1;
+		$self->seen($md5) and next;
+		print "$1\n";
+
+		# Fetch the document from the cache
 		my $document = $cache->get_document($md5);
 		unless ( $document ) {
 			warn("Failed to retrieve $md5 from the cache");
 			next;
 		}
 
-		$self->process_document($document, 'safe');
-		$done += (stat(($cache->_paths($md5))[1]))[7];
-		$rate = $done / (Time::HiRes::time() - $start);
-		$left = ($total - $done) / $rate;
-		next if $count % 100;
+		# Process the document
+		$self->process_document(
+			document => $document,
+			md5      => $md5,
+			hintsafe => 1,
+		);
+		next if ++$count % 100;
 		$self->commit_begin;
 	}
 	$self->commit;
+	print "\n";
 
 	# Add the indexes back to the database
 	$self->trace("Restoring indexes...");
@@ -294,11 +306,9 @@ sub process_file {
 		# If and only if every plugin has seen the document
 		# we can shortcut and don't need to load it.
 		my $md5 = PPI::Util::md5hex_file($path);
-		if ( $self->seen($md5) ) {
-			return 1;
-		}
+		return 1 if $self->seen($md5);
 	}
-	
+
 	# Load the document
 	my $document = PPI::Document->new( $path,
 		readonly => 1,
@@ -307,21 +317,28 @@ sub process_file {
 		 warn("Failed to parse '$path'");
 		 next;
 	}
-
-	$self->process_document($document);
+	$self->process_document(
+		document => $document,
+	);
 }
 
 # Forcefully process a docucment
 sub process_document {
 	my $self     = shift;
-	my $document = shift;
-	my $plugins  = $self->{plugins};
+	my %params   = (@_ > 1) ? @_ : ( document => $_[0] );
+	my $document = $params{document};
+	my $md5      = $params{md5} || $document->hex_id;
+	my $hintsafe = $params{hintsafe};
 
-	# Sort plugins with dustructive last
-	my @names = sort {
+	# Filter out plugins we don't need to rerun
+	# and sort plugins with destructive last
+	my $plugins = $self->{plugins};
+	my @names   = sort {
 		$plugins->{$a}->destructive <=> $plugins->{$b}->destructive
 		or
 		$a cmp $b
+	} grep {
+		not $self->seen($md5, $_)
 	} keys %$plugins;
 
 	# Create the plugin objects
@@ -332,9 +349,17 @@ sub process_document {
 		if ( $plugins->{$name}->destructive and $name ne $names[-1] ) {
 			# Run the plugin on a copy
 			my $copy = $document->clone;
-			$plugins->{$name}->process_document($copy, @_);
+			$plugins->{$name}->process_document(
+				document => $document,
+				md5      => $md5,
+				hintsafe => $hintsafe,
+			);
 		} else {
-			$plugins->{$name}->process_document($document, @_);
+			$plugins->{$name}->process_document(
+				document => $document,
+				md5      => $md5,
+				hintsafe => $hintsafe,
+			);
 		}
 	}
 
@@ -474,6 +499,35 @@ sub perl_modules {
 #####################################################################
 # Support Methods
 
+sub selectcol_index {
+	my ($dbh, $stmt, $attr, @bind) = @_;
+	my $sth = (ref $stmt) ? $stmt : $dbh->prepare($stmt, $attr);
+	return unless $sth;
+	$sth->execute(@bind) || return;
+	my $column = $attr->{Columns} ? $attr->{Columns}->[0] : 1;
+	my $value  = undef;
+	$sth->bind_col($column, \$value) || return;
+	my $row  = 0;
+	my %hash = ();
+	if ( my $max = $attr->{MaxRows} ) {
+		while ( $sth->fetch ) {
+			last if ++$row > $max;
+			$hash{$value} = 1;
+		}
+	} else {
+		while ( $sth->fetch ) {
+			$hash{$value} = 1;			
+		}
+	}
+	return \%hash;
+}
+
+sub in {
+	my $self = shift;
+	my $sql  = '( ' . join( ', ', map { '?' } @_ ) . ' )';
+	return ( $sql, @_ );
+}
+
 sub trace {
 	print STDERR map { "# $_\n" } @_[1..$#_];
 }
